#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include "hdfi.h"
#include "utilities.h"
#include "crisSisGetUtilities.h"

void getBeginEndSecsOfStructPriorTo(uint32 *startAndStop,
		uint32 secondsSince1996, char *dataFilePath, char *structName) {
    int ct = 0, de = 0;
    int day1996, index, fd;
    struct tm *pTimeStruct;
    uint32 dayStartSecond, secondsSince12AM, stamp;
    char *pBuffer;

	/* only command tables and diagnostic events should be calling */
	/* this routine. */
    if (strstr(structName, "CommandTable") != NULL) ct = 1;
    else if (strstr(structName, "DiagnosticEvent") != NULL) de = 1;
    else {
	    /* if we got here, invalid call */
	fprintf(stderr, "getStructSizeFromName: ERROR\n");
	fprintf(stderr, "  Invalid call to getBeginEndSecsOfStructPriorTo.\n");
	fprintf(stderr, "  Struct name ->%s<- not recognized.  Exiting.\n\n",
								structName);
	exit(0);
    }

    day1996 = getDay1996FromSecondsSince1996(secondsSince1996);
    pTimeStruct = getDateFromDay1996(day1996);
    dayStartSecond = getSecondsSince1996FromDate(pTimeStruct);
    secondsSince12AM = secondsSince1996 - dayStartSecond;

    if (!(fileExists(dataFilePath))) {
	    /* file does not exist */
	fprintf(stderr, "WARNING: File %s does not exist.\n", dataFilePath);
	fprintf(stderr, "No data that day? Checking previous days events.\n\n");
	index = -1;
    }
    else {
	    /* file does exist */
	int structCount, structSize, hasHeader;

	hasHeader = 1;
	structSize = getStructSizeFromName(structName);
	pBuffer = malloc(structSize);
	structCount = getFilesStructCount(dataFilePath, structSize, hasHeader);
	    /* initial guess */
	index = (int) (structCount * (secondsSince12AM / 86400.));
	/* fprintf(stderr, "index = %d\n", index); */

	fd = open(dataFilePath, O_RDONLY);	/* open to read */
	lseek(fd, HEADER_BYTES + (index * structSize), 0);
	readFromFile(fd, pBuffer, structSize);
	stamp = getTimeStamp(pBuffer, structName);
	/* fprintf(stderr, "1: stamp, index = %ld, %d\n", stamp, index); */

	    /* step forward past zeroed structs */
	while (stamp == 0) {
	    index++;
	    if (index >= structCount) break;
	    lseek(fd, HEADER_BYTES + (index * structSize), 0);
	    readFromFile(fd, pBuffer, structSize);
	    stamp = getTimeStamp(pBuffer, structName);
	    /* fprintf(stderr, "2: stamp, index = %ld, %d\n", stamp, index); */
	}

	if ((stamp != 0) && (stamp <= secondsSince1996)) {
		/* step forward past structs occuring before secondsSince1996 */
	    while (stamp <= secondsSince1996) {
		index++;
		if (index >= structCount) break;
		lseek(fd, HEADER_BYTES + (index * structSize), 0);
		readFromFile(fd, pBuffer, structSize);
		stamp = getTimeStamp(pBuffer, structName);
		/* fprintf(stderr, "3:stamp,index = %ld,%d\n", stamp, index); */
	    }

		/* step back one struct */
	    index--;
	    lseek(fd, HEADER_BYTES + (index * structSize), 0);
	    readFromFile(fd, pBuffer, structSize);
	    stamp = getTimeStamp(pBuffer, structName);
	    /* fprintf(stderr, "4: stamp, index = %ld, %d\n", stamp, index); */
		/* if stamp is 0, step back till finding non-zero stamp */
	    while (stamp ==0) {
		index--;
		if (index < 0) break;
		lseek(fd, HEADER_BYTES + (index * structSize), 0);
		readFromFile(fd, pBuffer, structSize);        
		stamp = getTimeStamp(pBuffer, structName);
		/* fprintf(stderr,"5: stamp, index = %ld, %d\n",stamp,index); */
	    }
	}
	else {
		/* step back before structs occuring later than */
		/* secondsSince1996.  also ignore stamps equal to zero. */
	    while ((stamp > secondsSince1996) || (stamp == 0)) {
		index--;
		if (index < 0) break;
		lseek(fd, HEADER_BYTES + (index * structSize), 0);
		readFromFile(fd, pBuffer, structSize);
		stamp = getTimeStamp(pBuffer, structName);
		/* fprintf(stderr,"6: stamp, index = %ld, %d\n",stamp,index); */
	    }
	}

	if (index >= 0) {
		/* prior non-zero struct lies in this day's structs (yeah!) */
	    lseek(fd, HEADER_BYTES + (index * structSize), 0);
	    readFromFile(fd, pBuffer, structSize);
	    startAndStop[0] = getTimeStamp(pBuffer, structName);
	    startAndStop[1] = getStopTimeStamp(pBuffer, structName);
	    close(fd);
	    return;
	}
	else {
		/* ugh! deal with looking in previous day's data file */
	    char newPath[256];
	    char instrumentID;
	    char suffix[10];

	    close(fd);

		/* determine path to previous day's file */
	    day1996--;
	    pTimeStruct = getDateFromDay1996(day1996);
	    instrumentID = getInstrumentIDFromStructName(structName);
	    getDataFileSuffixFromStructName(suffix, structName);
	    generateDataFilePath(newPath, pTimeStruct, instrumentID, suffix);

	    if (!(fileExists(newPath))) {
		startAndStop[0] = 0L;
		startAndStop[1] = 0L;
		return;
	    }

		/* look for last non-zero struct in previous day's file */
	    structCount = getFilesStructCount(newPath, structSize, hasHeader);
	    index = structCount - 1;
	    fd = open(newPath, O_RDONLY);
	    lseek(fd, HEADER_BYTES + (index * structSize), 0);
	    readFromFile(fd, pBuffer, structSize);
	    stamp = getTimeStamp(pBuffer, structName);
	    /* fprintf(stderr, "7: stamp, index = %ld, %d\n", stamp, index); */
		/* step back past zeroed structs */
	    while (stamp == 0) {
		index--;
		if (index < 0) break;
		lseek(fd, HEADER_BYTES + (index * structSize), 0);
		readFromFile(fd, pBuffer, structSize);
		stamp = getTimeStamp(pBuffer, structName);
		/* fprintf(stderr,"8: stamp, index = %ld, %d\n",stamp,index); */
	    }

	    if (index >= 0) {
		lseek(fd, HEADER_BYTES + (index * structSize), 0);
		readFromFile(fd, pBuffer, structSize);
		startAndStop[0] = getTimeStamp(pBuffer, structName);
		startAndStop[1] = getStopTimeStamp(pBuffer, structName);
		close(fd);
		return;
	    }
	    else {
		    /* didn't find a non-zero struct */
		startAndStop[0] = 0L;
		startAndStop[1] = 0L;
		close(fd);
		return;
	    }
	}
    }
}
