#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include "hdfi.h"
#include "utilities.h"
#include "crisSisGetUtilities.h"

void *getSubsetOfDaysStructs(int day1996, char *path, char *structName,
				uint32 getStampsAfter, uint32 getStampsUpTo) {
    char *pStructArrayStart, *pNextStructInArray, *dataBuffer, *pIntoDataBuffer;
    int structSize, hasHeader, structsInFileCount, fd, i;
    int firstValidDataBlockNumber, lastValidDataBlockNumber, maxValidDataBlocks;
    int startAndStopNumbers[2];
    int goodStructCount, doneReading;
    uint32 timeStamp;

    structSize = getStructSizeFromName(structName);
    hasHeader = 1;
    structsInFileCount = getFilesStructCount(path, structSize, hasHeader);

	/* if no structs were found, return pointer to single terminal struct */
    if (structsInFileCount == 0) {
	pStructArrayStart = (char *) calloc(1, structSize);
	memset(pStructArrayStart, (char) 0, structSize);  /* zero out struct */
	return pStructArrayStart;
    }

	/* open the data file for reading	*/
    if ((fd = open(path, O_RDONLY, 0)) == -1) {
	    /* can't open the file for some reason */
	char CANT_OPEN_ERROR_MSG[] = "Unable to open file %s\n\nExiting.\n\n";
	perror("getSubsetOfDaysStructs\n\t");
	fprintf(stderr, CANT_OPEN_ERROR_MSG, path);
	exit(1);
    }

	/* find the first and last data structs in the file with a time	*/
	/*     stamp later than getStampsAfter and before or equal to	*/
	/*     getStampsUpTo.						*/

    getDataBlockStartAndStopNumbers(day1996, fd, structName, getStampsAfter,
			getStampsUpTo, structsInFileCount, startAndStopNumbers);
    firstValidDataBlockNumber = startAndStopNumbers[0];
    lastValidDataBlockNumber = startAndStopNumbers[1];

	/* if no valid structs were found, return pointer to single	*/
	/*	terminal struct						*/
    if (firstValidDataBlockNumber < 0) {
	close(fd);
	pStructArrayStart = (char *) calloc(1, structSize);
	memset(pStructArrayStart, (char) 0, structSize);  /* zero out struct */
	return pStructArrayStart;
    }
    maxValidDataBlocks = lastValidDataBlockNumber+1 - firstValidDataBlockNumber;

	/* allocate memory for structs (+1 for terminal struct) */
	/*     dataBuffer is used to read everything that is possibly	*/
	/*     valid in one read.  pStructArrayStart is the array	*/
	/*     which will eventually be passed back after copying all	*/
	/*     valid structs from dataBuffer.				*/
    dataBuffer = (char *) calloc((maxValidDataBlocks + 1), structSize);
    pStructArrayStart = (char *) calloc((maxValidDataBlocks + 1), structSize);

	/* go to first valid data struct in file */
    lseek(fd, (HEADER_BYTES + (firstValidDataBlockNumber * structSize)), 0);
    readFromFile(fd, dataBuffer, (maxValidDataBlocks * structSize));

    goodStructCount = 0;
    doneReading = 0;
    pIntoDataBuffer = dataBuffer;
    pNextStructInArray = pStructArrayStart;
    for (i = 0; i < maxValidDataBlocks; i++) {
	timeStamp = getTimeStamp(pIntoDataBuffer, structName);
	if ((timeStamp <= getStampsAfter) || (timeStamp == 0xFFFFFFFF)) {
	    /* invalid struct (zeroed struct meaning no data		*/
	    /*     is archived in this slot or struct filled with	*/
	    /*     (char) -1s, meaning data is in the overlap file	*/
	    /*     here).  no copy.					*/
	}
	else {
	    memcpy(pNextStructInArray, pIntoDataBuffer, structSize);
	    pNextStructInArray += structSize;
	    goodStructCount++;
	}
	pIntoDataBuffer += structSize;
    }
    close(fd);
    free(dataBuffer);

	/* zero the terminal struct */
    memset(pNextStructInArray, (char) 0, structSize);

	/* cut struct array down to size */
    pStructArrayStart = realloc(pStructArrayStart,
				(goodStructCount + 1) * structSize);
    return pStructArrayStart;
}
