#include <stdio.h>
#include "hdfi.h"
#include "crisSisPutUtilities.h"
#include "utilities.h"

/*

createInitializedFile takes a file descriptor, a non-initialized data buffer,
the buffer's size, and the data type ID (e.g., 0x00000230 for
L1SisEventCycle) and initializes the day's data file for the particular
data type.  it writes an 8 word header file and zeros out the rest of the
file.  the file is made large enough on initialization to contain all
the data to be written to it for that day.

*/

void createInitializedFile(int fd, char dataBuffer[], int bufferSize,
							int dataTypeID) {
    unsigned int DATA_INTERVALS_PER_DAY = 24 * 3600 / DATA_BLOCK_INTERVAL + 1;

    int i;
    static int headerBuffer[HEADER_BYTES / sizeof(int)];

	/* create the header */
    headerBuffer[0] = PATTERN_WORD;
    headerBuffer[1] = dataTypeID;
	/* unused */
    for (i = 2; i < 8; i++) headerBuffer[i] = CLEARED_WORD;

	/* write the header */
    writeToFile(fd, (char *) headerBuffer, HEADER_BYTES);

	/* zero out the rest of the file */
    for (i = 0; i < bufferSize; i++) dataBuffer[i] = 0;
    for (i = 0; i < DATA_INTERVALS_PER_DAY; i++)
					writeToFile(fd, dataBuffer, bufferSize);
}
