#include <stdio.h>
#include "hdfi.h"
#include "utilities.h"
#include "crisSisPutUtilities.h"

/*

createInitializedFileWithIntervalN takes a file descriptor, a
non-initialized data buffer, the buffer's size, the data
type ID (e.g., 0x00000230 for L1SisEventCycle), and a time interval,
 and initializes the day's data file for the particular data type.
it writes an 8 word header file and zeros out the rest of the file.
the file is made large enough on initialization to contain all the
data to be written to it for that day.

the difference between createInitializedFile and
createInitializedFileWithIntervalN is that the latter is more
general.  createInitializedFile assumes a 256 second time interval
(which is the usual).

*/

void createInitializedFileWithIntervalN(int fd, char dataBuffer[],
			int bufferSize, int dataTypeID, int timeInterval) {
    unsigned int DATA_INTERVALS_PER_DAY;

    int i;
    static int headerBuffer[HEADER_BYTES / sizeof(int)];

    DATA_INTERVALS_PER_DAY = 24 * 3600 / timeInterval;
    if ((24 * 3600) % timeInterval) DATA_INTERVALS_PER_DAY++;

	/* create the header */
    headerBuffer[0] = PATTERN_WORD;
    headerBuffer[1] = dataTypeID;
	/* unused */
    for (i = 2; i < 8; i++) headerBuffer[i] = CLEARED_WORD;

	/* write the header */
    writeToFile(fd, (char *) headerBuffer, HEADER_BYTES);

	/* zero out the rest of the file */
    for (i = 0; i < bufferSize; i++) dataBuffer[i] = 0;
    for (i = 0; i < DATA_INTERVALS_PER_DAY; i++)
					writeToFile(fd, dataBuffer, bufferSize);
}
