#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "hdfi.h"
#include "sis.h"
#include "FileTypes.h"
#include "sisPutData.h"
#include "crisSisPutUtilities.h" 
#include "utilities.h"

/*

*/

static void localTerminationProcedure(char *errorMessage);


int putSisDiagnosticEvent(struct L1SisDiagnosticEvent *pDataStruct) {
	/** struct type dependent code **/
    unsigned int DATA_TYPE_ID = FILE_SIS_DIAGNOSTIC_EVENT;
    char FILE_TYPE_SUFFIX[] = "de";

	/* declared static so memory allocation occurs only once */
	/** struct type dependent code **/
    static char dataBuffer[sizeof(struct L1SisDiagnosticEvent)];
    char *pDataBuffer, *pPassedData;

    int fd, tempFd, i, intervalNumber, bytesRead;
    int foundIdenticalDataStructInFile;
    char path[256], tempPath[256], message[256];
    struct stat oldFileStats, newFileStats;

	/* intervalNumber is unused in putSisDiagnosticEvent */
	/** struct type dependent code **/
    determineDataFilePathAndCreateDirsAsNeeded(
		pDataStruct->Second1996MinorFrame0,
		FILE_TYPE_SUFFIX, path, &intervalNumber, 'S');

    pDataBuffer = dataBuffer;
    if ((fd = open(path, O_RDWR)) == -1) {	/* file doesn't exist yet? */
	fd = openOrCreateFileToReadWrite(path);
	writeHeader(fd, DATA_TYPE_ID);
	    /* since this is a new file, just write the struct and return */
	writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));
	close(fd);
	return 0;
    }

	/* file already exists.  first read the last struct in the file */
	/* and see if we just need to append the passed struct to file's end */

	/* go to end of file, and back up one struct size */
    lseek(fd, -1 * sizeof(dataBuffer), 2);
    readFromFile(fd, dataBuffer, sizeof(dataBuffer));
	/* is new struct's time stamp past last struct in file's time stamp? */
	/** struct type dependent code **/
    if (pDataStruct->Second1996MinorFrame0 >
	((struct L1SisDiagnosticEvent *) dataBuffer)->Second1996MinorFrame0) {
	    /* append passed struct and return */
	lseek(fd, 0L, 2);   /* go to end of file */
	writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));
	close(fd);
	return 0;
    }

	/* the passed struct needs to be inserted into place, so we need */
	/* to rewrite the file and stick it in where it belongs... */

	/* create new temp file */
    strcpy(tempPath, path);
    strcat(tempPath, ".tmp");
    tempFd = openOrCreateFileToReadWrite(tempPath);
    writeHeader(tempFd, DATA_TYPE_ID);

	/* read from original file */
    lseek(fd, HEADER_BYTES, 0);		/* go to start of file, past header */
	/* read in the next data structure from file */
    while ((bytesRead = read(fd, dataBuffer, sizeof(dataBuffer)))) {
	/** struct type dependent code **/
	if (pDataStruct->Second1996MinorFrame0 >
	  ((struct L1SisDiagnosticEvent *) dataBuffer)->Second1996MinorFrame0) {
		/* struct in file is earlier than passed struct, write it out */
	    writeToFile(tempFd, dataBuffer, sizeof(dataBuffer));
	}
	else if (pDataStruct->Second1996MinorFrame0 <
	  ((struct L1SisDiagnosticEvent *) dataBuffer)->Second1996MinorFrame0) {
		/* found where passed struct needs to be inserted.  write it */
	    writeToFile(tempFd, (char *) pDataStruct, sizeof(dataBuffer));
		/* now write struct it was compared to just now */
	    writeToFile(tempFd, dataBuffer, sizeof(dataBuffer));
		/* get out of comparison loop */
	    break;
	}
	else {
		/* time stamps are identical.  no write to original file, */
		/* but write passed struct to overlap file if it is */
		/* different than the struct in the file.  if the two */
		/* structs are identical, just clean up and return. */
	    close(fd);
	    close(tempFd);
	    if (remove(tempPath)) {
		sprintf(message, "Couldn't remove %s.\n", tempPath);
		localTerminationProcedure(message);
		return -1;
	    }

	    pPassedData = (char *) pDataStruct;
	    pDataBuffer = dataBuffer;
	    for (i = 0; i < sizeof(dataBuffer); i++) {
		if (*pPassedData == *pDataBuffer) {
			/* this comparison needs to be true over all chars */
		    foundIdenticalDataStructInFile = 1;
		    pPassedData++;
		    pDataBuffer++;
		}
		else {
			/* this only needs to happen once in a struct */
		    foundIdenticalDataStructInFile = 0;
		    break;
		}
	    }
	    if (foundIdenticalDataStructInFile) {
		return 3;
	    }
	    else {
		    /* append passed struct to an overlap file and return. */
		strcpy(tempPath, path);
		strcat(tempPath, ".overlap");
		tempFd = openOrCreateFileToReadWrite(tempPath);
		lseek(tempFd, 0L, 2);	/* go to end of file */
		writeToFile(tempFd, (char *) pDataStruct, sizeof(dataBuffer));
		close(tempFd);
		return 1;
	    }
	}
    }
	/* now write out the rest of the structs in the original file */
    while ((bytesRead = read(fd, dataBuffer, sizeof(dataBuffer))))
			writeToFile(tempFd, dataBuffer, sizeof(dataBuffer));
    close(fd);
    close(tempFd);

	/* do a simple error check to make sure new file is larger than */
	/* the original file at least... */
    stat(path, &oldFileStats);
    stat(tempPath, &newFileStats);
    if (oldFileStats.st_size >= newFileStats.st_size) {
	sprintf(message, "New file %s is not larger than old file %s.\n",
								tempPath, path);
	localTerminationProcedure(message);
	return -1;
    }
    else {
	    /* finally, replace the old file with the new file */
	if (remove(path)) {
	    sprintf(message, "Couldn't remove %s.\n", path);
	    localTerminationProcedure(message);
	    return -1;
	}

	if (rename(tempPath, path)) {
	    sprintf(message, "Couldn't rename %s to %s.\n", tempPath, path);
	    localTerminationProcedure(message);
	    return -1;
	}
    }
    return 0;
}

static void localTerminationProcedure(char *errorMessage) {
    perror("putSisDiagnosticEvent\n\t");
    fprintf(stderr, "%s\n", errorMessage);
    if (getAppTerminatePermission()) {
	fprintf(stderr, "\n\tExiting.\n\n");
	exit(1);
    }
    else fprintf(stderr, "\n\tSERIOUS ERROR!\n\n");
}
