#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "utilities.h"

/*

generateDataFilePath takes the number of seconds
since 00:00:00 1/1/96, a data type file suffix, and returns the path
for the data file (of the form .../S1996.01/S1996.01.02/S1996.01.02.evt)

*/

    /* procedure to use in case of error (defined below) */
static void localTerminationProcedure(char *pathBuffer);


void generateDataFilePath(char *pathBuffer, struct tm *timeStruct,
				char instrumentID, const char *dataFileSuffix) {
    int year, month, mday;
    char yearMonthString[8], dateString[10];
    char *BASE_DIRECTORY, *pChar, *pToken;
    char baseDirShort[256], buffer[256];

    if ((instrumentID == 'C') || (instrumentID == 'c')) {	/* CRIS */
	instrumentID = 'C';
	BASE_DIRECTORY = getenv("L1_CRIS_DATA_BASE_DIRECTORY");
	if (BASE_DIRECTORY == NULL) {
	    fprintf(stderr, "The environment variable ");
	    fprintf(stderr, "L1_CRIS_DATA_BASE_DIRECTORY");
	    fprintf(stderr, "\nhas not been defined.\n");
	    localTerminationProcedure(pathBuffer);
	    return;
	}
    }
    else if ((instrumentID == 'S') || (instrumentID == 's')) {       /* SIS */
	instrumentID = 'S';
	BASE_DIRECTORY = getenv("L1_SIS_DATA_BASE_DIRECTORY");
	if (BASE_DIRECTORY == NULL) {
	    fprintf(stderr, "The environment variable ");
	    fprintf(stderr, "L1_SIS_DATA_BASE_DIRECTORY");
	    fprintf(stderr, "\nhas not been defined.\n");
	    localTerminationProcedure(pathBuffer);
	    return;
	}
    }
    else {
	fprintf(stderr, "Invalid instrument ID: %c.\n", instrumentID);
	localTerminationProcedure(pathBuffer);
	return;
    }
    if (!directoryExists(BASE_DIRECTORY)) {
	fprintf(stderr, "The base directory %s does not exist.\n",
								BASE_DIRECTORY);
	localTerminationProcedure(pathBuffer);
	return;
    }

        /* use directories after the "data" directory in the environment */
        /* variable L1_[CRIS|SIS]_BASE_DIRECTORY as a file identifier to */
        /* be prepended to the data file name (baseDirShort refers to the */
        /* base directory, shortened by chopping of everything up to "data") */
    strcpy(buffer, BASE_DIRECTORY);
    pChar = buffer;
    strcpy(baseDirShort, "");
    while ((pToken = strtok(pChar, "/")) != NULL) {
        pChar = NULL;
        if (!strcmp(pToken, "data")) {  /* if (token == "data") */
            /* while (pToken = strtok(pChar, "/")) { */
            while ((pToken = strtok(pChar, "/")) != NULL) {
                if (!strcmp(pToken, "data")) {
                    fprintf(stderr,
                        "Directory \"data\" appears multiple times in base");
                    fprintf(stderr, "directory \"%s\"\n", BASE_DIRECTORY);
		    localTerminationProcedure(pathBuffer);
		    return;
                }
                if (!strcmp(pToken, "")) break;         /* ignore nothing */
                if (*pToken == ' ') break;              /* ignore spaces */
                if (*pToken == '	') break;       /* ignore tabs */
                strcat(baseDirShort, pToken);
                strcat(baseDirShort, ".");
            }
            break;
        }
    }
    if (!strcmp(baseDirShort, "")) {
        fprintf(stderr, "\"data\" directory and/or its subdirectories not");
        fprintf(stderr, " found in base directory \"%s\"\n", BASE_DIRECTORY);
	localTerminationProcedure(pathBuffer);
	return;
    }

    year = timeStruct->tm_year + 1900;
    month = timeStruct->tm_mon + 1;
    mday = timeStruct->tm_mday;
 
    sprintf(yearMonthString, "%4.4u.%2.2u", year, month);
    sprintf(dateString, "%4.4u.%2.2u.%2.2u", year, month, mday);
    sprintf(pathBuffer, "%s/%s/%s", BASE_DIRECTORY,yearMonthString,dateString);

    if (!directoryExists(pathBuffer)) {
	fprintf(stderr, "WARNING: Directory %s does not exist.\n\n",pathBuffer);
    }
 
    sprintf(pathBuffer, "%s/%s/%s/%s%s.%s", BASE_DIRECTORY,
	yearMonthString, dateString, baseDirShort, dateString, dataFileSuffix);
}

static void localTerminationProcedure(char *pathBuffer) {
    if (getAppTerminatePermission()) {
	fprintf(stderr, "\n\tExiting.\n\n");
	exit(1);
    }
    else {
	fprintf(stderr, "\n\tSERIOUS ERROR!\n\n");
	sprintf(pathBuffer, "UNDEFINED_DATA_FILE_PATH");
    }
}
