#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "utilities.h"

/*

*/

    /* procedure to use in case of error (defined below) */
static void localTerminationProcedure(struct tm *date);


void getFirstDateOfCurrentArchivesData(char instrumentID, struct tm *date) {
    int year, month, mday;
    char yearMonthString[8], dateString[10];
    char *BASE_DIRECTORY = "";
    char pathBuffer[256];

    if ((instrumentID == 'C') || (instrumentID == 'c')) {	/* CRIS */
	instrumentID = 'C';
	BASE_DIRECTORY = getenv("L1_CRIS_DATA_BASE_DIRECTORY");
	if (BASE_DIRECTORY == NULL) {
	    fprintf(stderr, "The environment variable ");
	    fprintf(stderr, "L1_CRIS_DATA_BASE_DIRECTORY");
	    fprintf(stderr, "\nhas not been defined.\n");
	    localTerminationProcedure(date);
	    return;
	}
    }
    else if ((instrumentID == 'S') || (instrumentID == 's')) {       /* SIS */
	instrumentID = 'S';
	BASE_DIRECTORY = getenv("L1_SIS_DATA_BASE_DIRECTORY");
	if (BASE_DIRECTORY == NULL) {
	    fprintf(stderr, "The environment variable ");
	    fprintf(stderr, "L1_SIS_DATA_BASE_DIRECTORY");
	    fprintf(stderr, "\nhas not been defined.\n");
	    localTerminationProcedure(date);
	    return;
	}
    }
    else {
	fprintf(stderr, "Invalid instrument ID: %c.\n", instrumentID);
	localTerminationProcedure(date);
	return;
    }
    if (!directoryExists(BASE_DIRECTORY)) {
	fprintf(stderr, "The base directory %s does not exist.\n",
								BASE_DIRECTORY);
	localTerminationProcedure(date);
	return;
    }

    for (year = 1996; year < 2100; year++) {
	for (month = 1; month <= 12; month++) {
	    sprintf(yearMonthString, "%4.4u.%2.2u", year, month);
	    sprintf(pathBuffer, "%s/%s", BASE_DIRECTORY, yearMonthString);
	    if (directoryExists(pathBuffer)) {
		for (mday = 1; mday <= 31; mday++) {
		    sprintf(dateString, "%4.4u.%2.2u.%2.2u", year, month, mday);
		    sprintf(pathBuffer, "%s/%s/%s", BASE_DIRECTORY,
						yearMonthString, dateString);
		    if (directoryExists(pathBuffer)) {
			date->tm_year = year - 1900;
			date->tm_mon = month - 1;
			date->tm_mday = mday;
			date->tm_hour = 0;
			date->tm_min = 0;
			date->tm_sec = 0;
			date->tm_isdst = 0;

			return;
		    }
		}
	    }
	}
    }

    date->tm_year = 0;
    date->tm_mon = 0;
    date->tm_mday = 0;
    date->tm_hour = 0;
    date->tm_min = 0;
    date->tm_sec = 0;
    date->tm_isdst = 0;
}


static void localTerminationProcedure(struct tm *date) {
    if (getAppTerminatePermission()) {
	fprintf(stderr, "\n\tExiting.\n\n");
	exit(1);
    }
    else {
	fprintf(stderr, "\n\tSERIOUS ERROR!\n\n");
	date->tm_sec = 0; date->tm_min = 0; date->tm_hour = 0;
	date->tm_mday = 0; date->tm_mon = 0; date->tm_year = 0;
	date->tm_wday = 0; date->tm_yday = 0; date->tm_isdst = 0;
    }
}
