/* $Id: initTabSec1996.c,v 1.1 1998/07/15 18:24:24 bruce Exp $ */

/* $RCSfile: initTabSec1996.c,v $ */
/* $Author: bruce $ */
/* $Date: 1998/07/15 18:24:24 $ */
/* $Revision: 1.1 $ */

/* This function initializes the month table that is used in the     */
/* conversion to and from seconds of 1996.  The month table is a     */
/* table that contains the number of seconds (from 00:00:00          */
/* January 1996) that have elapsed at the start of the month.  The   */
/* month table covers the range 1996-2040.                           */
/*                                                                   */
/* Note: This function includes the leap second table for the        */
/*       period of 1996-2040.  If there are any updates to this      */
/*       table, the function should be recomplied.                   */
/*                                                                   */
/* Note: Leap second for June 1997.                                  */
/*                                                                   */
/* Arguments                                                         */
/*    MonthTable = month table - output                              */

#include <sys/types.h>

void initTabSec1996(u_long *MonthTable)
{
   /* Leap seconds at the end of June and December for 1996-2040     */

   u_long LeapSecondTable[] =
                   {       0L,0L, 1L,0L, 0L,0L, 0L,0L,  /* 1996-1999 */
                    0L,0L, 0L,0L, 0L,0L, 0L,0L, 0L,0L,  /* 2000-2004 */
                    0L,0L, 0L,0L, 0L,0L, 0L,0L, 0L,0L,  /* 2005-2009 */
                    0L,0L, 0L,0L, 0L,0L, 0L,0L, 0L,0L,  /* 2010-2014 */
                    0L,0L, 0L,0L, 0L,0L, 0L,0L, 0L,0L,  /* 2015-2019 */
                    0L,0L, 0L,0L, 0L,0L, 0L,0L, 0L,0L,  /* 2020-2024 */
                    0L,0L, 0L,0L, 0L,0L, 0L,0L, 0L,0L,  /* 2025-2029 */
                    0L,0L, 0L,0L, 0L,0L, 0L,0L, 0L,0L,  /* 2030-2034 */
                    0L,0L, 0L,0L, 0L,0L, 0L,0L, 0L,0L,  /* 2035-2039 */
                    0L,0L                            }; /* 2040      */

      int i;
      int month;
   u_long TotalLeapSeconds;
      int year;

   /* Generate a table of months in which each entry is the number */
   /* of days since January 1, 1996.                               */
   
   MonthTable[ 0] = (u_long) (0);
   MonthTable[ 1] = (u_long) (0+31);
   MonthTable[ 2] = (u_long) (0+31+28);
   MonthTable[ 3] = (u_long) (0+31+28+31);
   MonthTable[ 4] = (u_long) (0+31+28+31+30);
   MonthTable[ 5] = (u_long) (0+31+28+31+30+31);
   MonthTable[ 6] = (u_long) (0+31+28+31+30+31+30);
   MonthTable[ 7] = (u_long) (0+31+28+31+30+31+30+31);
   MonthTable[ 8] = (u_long) (0+31+28+31+30+31+30+31+31);
   MonthTable[ 9] = (u_long) (0+31+28+31+30+31+30+31+31+30);
   MonthTable[10] = (u_long) (0+31+28+31+30+31+30+31+31+30+31);
   MonthTable[11] = (u_long) (0+31+28+31+30+31+30+31+31+30+31+30);
   for (month=0; month<12; month++)
       MonthTable[month]++;
   month = 0;
   for (year=1997; year<=2040; year++) {
      if (year % 4) 
         for (i=month; i<month+12; i++)
             MonthTable[i+12] = MonthTable[i] + 365L;
      else
         for (i=month; i<month+12; i++)
             MonthTable[i+12] = MonthTable[i] + 366L;
      month += 12;
   }
   month = 0;
   for (year=1996; year<=2040; year+=4) {
      MonthTable[month]--;
      MonthTable[month+1]--;
      month += 48;
   }
   
   /* Convert the table of months from units of days to seconds and */
   /* add in any necessary leap seconds.                            */
   
   TotalLeapSeconds = 0L;
   for (year=1996; year<=2040; year++) {
      for (i=0; i<6; i++) {
         month = 12 * (year -1996) + i;
         MonthTable[month] = 86400L * MonthTable[month] +
                                                     TotalLeapSeconds;
      }
      TotalLeapSeconds += LeapSecondTable[2*(year-1996)];
      for (i=6; i<12; i++) {
         month = 12 * (year -1996) + i;
         MonthTable[month] = 86400L * MonthTable[month] +
                                                     TotalLeapSeconds;
      }
      TotalLeapSeconds += LeapSecondTable[2*(year-1996)+1];
   }
}
